@@ -1,6 +1,6 @@
-/**
+/*
  * Copyright (C) 2007-2009 Sergio Pistone <sergio_pistone@yahoo.com.ar>
- * Copyright (C) 2010-2015 Mladen Milinkovic <max@smoothware.net>
+ * Copyright (C) 2010-2017 Mladen Milinkovic <max@smoothware.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -241,7 +241,7 @@ Subtitle::framesPerSecond() const
 void
 Subtitle::setFramesPerSecond(double framesPerSecond)
 {
-	if(m_framesPerSecond != framesPerSecond)
+	if(m_framesPerSecond != framesPerSecond) // FIXME: Is this check important (floating point comparsion is unsafe)?
 		processAction(new SetFramesPerSecondAction(*this, framesPerSecond));
 }
 
@@ -296,15 +296,13 @@ Subtitle::lastIndex() const
 SubtitleLine *
 Subtitle::line(int index)
 {
-//      return (index < m_lines.count() ? m_lines.at( index ) : 0);
-	return index < 0 ? 0 : (index < m_lines.count() ? m_lines.at(index) : 0);
+	return index < 0 ? nullptr : (index < m_lines.count() ? m_lines.at(index) : nullptr);
 }
 
 const SubtitleLine *
 Subtitle::line(int index) const
 {
-//      return (index < m_lines.count() ? m_lines.at( index ) : 0);
-	return index < 0 ? 0 : (index < m_lines.count() ? m_lines.at(index) : 0);
+	return index < 0 ? nullptr : (index < m_lines.count() ? m_lines.at(index) : nullptr);
 }
 
 SubtitleLine *
@@ -388,9 +386,6 @@ Subtitle::removeAllAnchors()
 void
 Subtitle::insertLine(SubtitleLine *line, int index)
 {
-	if(index > 0)
-		setLastValidCachedIndex(index - 1);
-
 	QList<SubtitleLine *> lines;
 	lines.append(line);
 	insertLines(lines, index);
@@ -404,6 +399,9 @@ Subtitle::insertLines(const QList<SubtitleLine *> &lines, int index)
 	if(index < 0)
 		index = m_lines.count();
 
+	if(m_lastValidCachedIndex >= index)
+		setLastValidCachedIndex(index - 1);
+
 	processAction(new InsertLinesAction(*this, lines, index));
 }
 
@@ -421,21 +419,21 @@ Subtitle::insertNewLine(int index, bool timeAfter, TextTarget target)
 	if(timeAfter) {
 		if(newLineIndex) {              // there is a previous line
 			SubtitleLine *prevLine = m_lines.value(newLineIndex - 1);
-			newLine->setTimes(prevLine->hideTime() + 100, prevLine->hideTime() + 1000);
+			newLine->setTimes(prevLine->hideTime() + 100., prevLine->hideTime() + 1000.);
 		} else if(newLineIndex < m_lines.count()) {     // there is a next line
 			SubtitleLine *nextLine = m_lines.value(newLineIndex);
-			newLine->setTimes(nextLine->showTime() - 1100, nextLine->showTime() - 100);
+			newLine->setTimes(nextLine->showTime() - 1100., nextLine->showTime() - 100.);
 		} else
-			newLine->setHideTime(1000);
+			newLine->setHideTime(1000.);
 	} else {                                        // ! timeAfter
 		if(newLineIndex < m_lines.count()) {    // there is a next line
 			SubtitleLine *nextLine = m_lines.at(newLineIndex);
-			newLine->setTimes(nextLine->showTime() - 1100, nextLine->showTime() - 100);
+			newLine->setTimes(nextLine->showTime() - 1100., nextLine->showTime() - 100.);
 		} else if(newLineIndex) {       // there is a previous line
 			SubtitleLine *prevLine = m_lines.at(newLineIndex - 1);
-			newLine->setTimes(prevLine->hideTime() + 100, prevLine->hideTime() + 1000);
+			newLine->setTimes(prevLine->hideTime() + 100., prevLine->hideTime() + 1000.);
 		} else
-			newLine->setHideTime(1000);
+			newLine->setHideTime(1000.);
 	}
 
 	if(target == Both || index == m_lines.count()) {
@@ -528,14 +526,14 @@ Subtitle::removeLines(const RangeList &r, TextTarget target)
 		Range lastRange = mutableRanges.last();
 		int lastIndex = lastRange.end() == linesCount - 1 ? lastRange.start() - 1 : linesCount - 1;
 		SubtitleLine *lastLine = lastIndex < linesCount ? m_lines.at(lastIndex) : 0;
-		Time showTime(lastLine ? lastLine->hideTime() + 100 : 0);
-		Time hideTime(showTime + 1000);
+		Time showTime(lastLine ? lastLine->hideTime() + 100. : Time());
+		Time hideTime(showTime + 1000.);
 
 		QList<SubtitleLine *> lines;
 		for(int index = 0, size = ranges.indexesCount(); index < size; ++index) {
 			lines.append(new SubtitleLine(SString(), SString(), showTime, hideTime));
-			showTime.shift(1100);
-			hideTime.shift(1100);
+			showTime.shift(1100.);
+			hideTime.shift(1100.);
 		}
 
 		processAction(new InsertLinesAction(*this, lines));
@@ -572,44 +570,68 @@ Subtitle::splitLines(const RangeList &ranges)
 
 	for(SubtitleIterator it(*this, ranges, true); it.current(); --it) {
 		SubtitleLine *line = it.current();
-		if(line->primaryLines() > 1) {
-			line->simplifyTextWhiteSpace(SubtitleLine::Primary);
 
-			long autoDurationsSum = 0;
-			QList<int> autoDurations;
-			SStringList primaryLines = line->primaryText().split('\n');
-			for(SStringList::ConstIterator ptIt = primaryLines.begin(), ptEnd = primaryLines.end(); ptIt != ptEnd; ++ptIt) {
-				const Time &autoDuration = SubtitleLine::autoDuration((*ptIt).string(), 60, 50,
-																	  50);
-				autoDurations.append(autoDuration.toMillis());
-				autoDurationsSum += autoDuration.toMillis();
-			}
+		line->simplifyTextWhiteSpace(SubtitleLine::Primary);
+		SString primaryText = line->primaryText();
+		if(primaryText.isEmpty())
+			continue;
 
-			double factor = (double)line->durationTime().toMillis() / autoDurationsSum;
-
-			SStringList secondaryLines = line->secondaryText().split('\n');
-			while(secondaryLines.count() < primaryLines.count())
-				secondaryLines.append(SString());
-			while(secondaryLines.count() > primaryLines.count()) {
-				SString lastLine = secondaryLines.last();
-				secondaryLines.pop_back();
-				secondaryLines.last() += '\n';
-				secondaryLines.last() += lastLine;
-			}
-
-			int subLineIndex = it.index(), splitLineIndex = 0;
-			for(SStringList::ConstIterator ptIt = primaryLines.begin(), ptEnd = secondaryLines.end(), stIt = secondaryLines.begin(), stEnd = secondaryLines.end(); ptIt != ptEnd && stIt != stEnd; ++ptIt, ++stIt, ++subLineIndex, ++splitLineIndex) {
-				if(splitLineIndex) {
-					SubtitleLine *newLine = new SubtitleLine();
-					newLine->setShowTime(line->hideTime() + 1);
-					insertLine(newLine, subLineIndex);
-					line = newLine;
+		if(line->primaryLines() < 2) {
+			if(primaryText.count(QChar::Space) == 0)
+				continue;
+			int len = primaryText.length();
+			int i = len / 2;
+			int j = i + len % 2;
+			for(; ; i--, j++) {
+				if(primaryText[i] == QChar::Space) {
+					primaryText[i] = QChar::LineFeed;
+					break;
+				}
+				Q_ASSERT(j <= len);
+				if(primaryText[j] == QChar::Space) {
+					primaryText[j] = QChar::LineFeed;
+					break;
+				}
+				if(i == 0) {
+					primaryText.append(QChar::LineFeed);
+					break;
 				}
-
-				line->setTexts(*ptIt, *stIt);
-				line->setDurationTime(Time((factor * autoDurations[splitLineIndex]) - 1.0));
 			}
 		}
+
+		double autoDurationsSum = 0;
+		QList<double> autoDurations;
+		SStringList primaryLines = primaryText.split('\n');
+		for(SStringList::ConstIterator ptIt = primaryLines.begin(), ptEnd = primaryLines.end(); ptIt != ptEnd; ++ptIt) {
+			const Time &autoDuration = SubtitleLine::autoDuration(ptIt->string(), 60, 50, 50);
+			autoDurations.append(autoDuration.toMillis());
+			autoDurationsSum += autoDuration.toMillis();
+		}
+
+		double factor = (line->durationTime().toMillis() + 1.) / autoDurationsSum;
+
+		SStringList secondaryLines = line->secondaryText().split('\n');
+		while(secondaryLines.count() < primaryLines.count())
+			secondaryLines.append(SString());
+		while(secondaryLines.count() > primaryLines.count()) {
+			SString lastLine = secondaryLines.last();
+			secondaryLines.pop_back();
+			secondaryLines.last() += '\n';
+			secondaryLines.last() += lastLine;
+		}
+
+		int subLineIndex = it.index(), splitLineIndex = 0;
+		for(SStringList::ConstIterator ptIt = primaryLines.begin(), ptEnd = secondaryLines.end(), stIt = secondaryLines.begin(), stEnd = secondaryLines.end(); ptIt != ptEnd && stIt != stEnd; ++ptIt, ++stIt, ++subLineIndex, ++splitLineIndex) {
+			if(splitLineIndex) {
+				SubtitleLine *newLine = new SubtitleLine();
+				newLine->setShowTime(line->hideTime() + 1.);
+				insertLine(newLine, subLineIndex);
+				line = newLine;
+			}
+
+			line->setTexts(*ptIt, *stIt);
+			line->setDurationTime(Time((factor * autoDurations[splitLineIndex]) - 1.0));
+		}
 	}
 
 	endCompositeAction();
@@ -666,12 +688,12 @@ Subtitle::shiftAnchoredLine(SubtitleLine *anchoredLine, const Time &newShowTime)
 	if(m_anchoredLines.indexOf(anchoredLine) == -1 || m_lines.isEmpty())
 		return;
 
-	const SubtitleLine *prevAnchor = NULL;
-	const SubtitleLine *nextAnchor = NULL;
+	const SubtitleLine *prevAnchor = nullptr;
+	const SubtitleLine *nextAnchor = nullptr;
 	foreach(auto anchor, m_anchoredLines) {
-		if((prevAnchor == NULL || prevAnchor->m_showTime < anchor->m_showTime) && anchor->m_showTime < anchoredLine->m_showTime)
+		if((prevAnchor == nullptr || prevAnchor->m_showTime < anchor->m_showTime) && anchor->m_showTime < anchoredLine->m_showTime)
 			prevAnchor = anchor;
-		if((nextAnchor == NULL || nextAnchor->m_showTime > anchor->m_showTime) && anchor->m_showTime > anchoredLine->m_showTime)
+		if((nextAnchor == nullptr || nextAnchor->m_showTime > anchor->m_showTime) && anchor->m_showTime > anchoredLine->m_showTime)
 			nextAnchor = anchor;
 	}
 	if((prevAnchor && prevAnchor->m_showTime > newShowTime) || (nextAnchor && nextAnchor->m_showTime < newShowTime))
@@ -704,10 +726,10 @@ Subtitle::shiftAnchoredLine(SubtitleLine *anchoredLine, const Time &newShowTime)
 			double scaleFactor = (newShowTime.toMillis() - prevAnchor->m_showTime.toMillis()) / (savedShowTime.toMillis() - prevAnchor->m_showTime.toMillis());
 			lastShowTime = scaleFactor * (last->m_showTime.toMillis() - prevAnchor->m_showTime.toMillis()) + prevAnchor->m_showTime.toMillis();
 		} else {
-			last = NULL;
+			last = nullptr;
 			lastShowTime = 0;
 		}
-		if(newShowTime.toMillis() < lastShowTime) {
+		if(newShowTime.toMillis() < lastShowTime && anchoredLine != last) {
 			anchoredLine->m_showTime = savedShowTime;
 			anchoredLine->m_hideTime = savedHideTime;
 			adjustLines(Range(anchoredLine->index(), last->index()), newShowTime.toMillis(), lastShowTime);
