@@ -1,6 +1,6 @@
-/**
+/*
  * Copyright (C) 2007-2009 Sergio Pistone <sergio_pistone@yahoo.com.ar>
- * Copyright (C) 2010-2015 Mladen Milinkovic <max@smoothware.net>
+ * Copyright (C) 2010-2017 Mladen Milinkovic <max@smoothware.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,9 +43,9 @@ SubtitleLine::ErrorFlag
 SubtitleLine::errorFlag(SubtitleLine::ErrorID id)
 {
 	if(id < 0 || id >= ErrorSIZE)
-		return (ErrorFlag)0;
+		return static_cast<ErrorFlag>(0);
 
-	return (ErrorFlag)(0x1 << id);
+	return static_cast<ErrorFlag>(0x1 << id);
 }
 
 SubtitleLine::ErrorID
@@ -54,8 +54,8 @@ SubtitleLine::errorID(SubtitleLine::ErrorFlag flag)
 	if(flag < 1)
 		return ErrorUNKNOWN;
 
-	int id = log2(static_cast<int>(flag));
-	return id < ErrorSIZE ? (ErrorID)id : ErrorUNKNOWN;
+	int id = static_cast<int>(log2(static_cast<int>(flag)));
+	return id < ErrorSIZE ? static_cast<ErrorID>(id) : ErrorUNKNOWN;
 }
 
 /// ERRORS DESCRIPTIONS
@@ -500,7 +500,7 @@ SubtitleLine::showTime() const
 }
 
 void
-SubtitleLine::setShowTime(const Time &showTime)
+SubtitleLine::setShowTime(const Time &showTime, bool safe/*=false*/)
 {
 	if(m_showTime == showTime)
 		return;
@@ -510,7 +510,10 @@ SubtitleLine::setShowTime(const Time &showTime)
 		m_subtitle->shiftAnchoredLine(this, showTime);
 		m_subtitle->endCompositeAction();
 	} else {
-		processAction(new SetLineShowTimeAction(*this, showTime));
+		if(safe && showTime > m_hideTime)
+			setTimes(m_hideTime, showTime);
+		else
+			processAction(new SetLineShowTimeAction(*this, showTime));
 	}
 }
 
@@ -521,9 +524,14 @@ SubtitleLine::hideTime() const
 }
 
 void
-SubtitleLine::setHideTime(const Time &hideTime)
+SubtitleLine::setHideTime(const Time &hideTime, bool safe/*=false*/)
 {
-	if(m_hideTime != hideTime)
+	if(m_hideTime == hideTime)
+		return;
+
+	if(safe && m_showTime > hideTime)
+		setTimes(hideTime, m_showTime);
+	else
 		processAction(new SetLineHideTimeAction(*this, hideTime));
 }
 
