@@ -1,9 +1,9 @@
 #ifndef SUBTITLELINE_H
 #define SUBTITLELINE_H
 
-/**
+/*
  * Copyright (C) 2007-2009 Sergio Pistone <sergio_pistone@yahoo.com.ar>
- * Copyright (C) 2010-2015 Mladen Milinkovic <max@smoothware.net>
+ * Copyright (C) 2010-2017 Mladen Milinkovic <max@smoothware.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,11 +21,6 @@
  * Boston, MA 02110-1301, USA.
  */
 
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "action.h"
 #include "sstring.h"
 #include "time.h"
@@ -171,10 +166,10 @@ public:
 	void simplifyTextWhiteSpace(TextTarget target);
 
 	Time showTime() const;
-	void setShowTime(const Time &showTime);
+	void setShowTime(const Time &showTime, bool safe=false);
 
 	Time hideTime() const;
-	void setHideTime(const Time &hideTime);
+	void setHideTime(const Time &hideTime, bool safe=false);
 
 	Time durationTime() const;
 	void setDurationTime(const Time &durationTime);
