@@ -1,9 +1,9 @@
 #ifndef APPLICATION_H
 #define APPLICATION_H
 
-/**
+/*
  * Copyright (C) 2007-2009 Sergio Pistone <sergio_pistone@yahoo.com.ar>
- * Copyright (C) 2010-2015 Mladen Milinkovic <max@smoothware.net>
+ * Copyright (C) 2010-2017 Mladen Milinkovic <max@smoothware.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,10 +21,6 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "mainwindow.h"
 #include "../core/subtitle.h"
 #include "../formats/format.h"
@@ -90,7 +86,7 @@ public:
 	void loadConfig();
 	void saveConfig();
 
-	QAction * action(const char *actionName);
+	QAction * action(const char *actionName) const;
 
 	/**
 	 * @brief triggerAction
@@ -203,9 +199,13 @@ public slots:
 
 	void seekBackwards();
 	void seekForwards();
+	void playOnlyCurrentLine();
 	void seekToPrevLine();
 	void seekToNextLine();
 
+	void playrateIncrease();
+	void playrateDecrease();
+
 	void setCurrentLineShowTimeFromVideo();
 	void setCurrentLineHideTimeFromVideo();
 
@@ -216,10 +216,6 @@ public slots:
 	void adjustToVideoPositionAnchorLast();
 	void adjustToVideoPositionAnchorFirst();
 
-	void setCurrentLineShowTimeFromWaveform();
-	void setCurrentLineHideTimeFromWaveform();
-	void insertLineFromWaveform();
-
 signals:
 	void subtitleOpened(Subtitle *subtitle);
 	void subtitleClosed();
