@@ -1,6 +1,6 @@
-/**
+/*
  * Copyright (C) 2007-2009 Sergio Pistone <sergio_pistone@yahoo.com.ar>
- * Copyright (C) 2010-2015 Mladen Milinkovic <max@smoothware.net>
+ * Copyright (C) 2010-2017 Mladen Milinkovic <max@smoothware.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -227,8 +227,8 @@ CurrentLineWidget::setCurrentLine(SubtitleLine *line)
 		connect(m_currentLine, SIGNAL(hideTimeChanged(const Time &)), this, SLOT(onLineHideTimeChanged(const Time &)));
 	}
 
-	onLineShowTimeChanged(m_currentLine ? m_currentLine->showTime() : 0);
-	onLineHideTimeChanged(m_currentLine ? m_currentLine->hideTime() : 0);
+	onLineShowTimeChanged(m_currentLine ? m_currentLine->showTime() : Time());
+	onLineHideTimeChanged(m_currentLine ? m_currentLine->hideTime() : Time());
 
 	if(m_currentLine) {
 		onLinePrimaryTextChanged(m_currentLine->primaryText());
@@ -439,6 +439,7 @@ CurrentLineWidget::onLineShowTimeChanged(const Time &showTime)
 	if(m_updateControls) {
 		m_updateCurrentLine = false;
 		m_showTimeEdit->setValue(showTime.toMillis());
+		m_hideTimeEdit->setMinimumTime(QTime(0, 0, 0, 0).addMSecs(showTime.toMillis()));
 		m_updateCurrentLine = true;
 	}
 }
