@@ -1,6 +1,6 @@
-/**
+/*
  * Copyright (C) 2007-2009 Sergio Pistone <sergio_pistone@yahoo.com.ar>
- * Copyright (C) 2010-2015 Mladen Milinkovic <max@smoothware.net>
+ * Copyright (C) 2010-2017 Mladen Milinkovic <max@smoothware.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,10 +18,6 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "application.h"
 #include "mainwindow.h"
 #include "../common/commondefs.h"
@@ -34,8 +30,11 @@
 #include <QCommandLineParser>
 #include <QCommandLineOption>
 
-#if defined HAVE_XINE && !defined HAVE_XCB
-#include <X11/Xlib.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+# if defined HAVE_XINE && !defined HAVE_XCB
+#  include <X11/Xlib.h>
+# endif
 #endif
 
 int
@@ -47,21 +46,24 @@ main(int argc, char **argv)
 
 	SubtitleComposer::Application app(argc, argv);
 
+	// hack(?) to find custom icons outside kde
+	QIcon::setThemeSearchPaths(QIcon::themeSearchPaths() << QStringLiteral(CUSTOM_ICON_INSTALL_PATH));
+
 	KLocalizedString::setApplicationDomain("subtitlecomposer");
 
 	KAboutData aboutData(
 		"subtitlecomposer",
 		i18n("Subtitle Composer"),
-		"0.6.4",
+		"0.6.5",
 		i18n("A KDE subtitle editor."),
 		KAboutLicense::GPL,
-		"&copy; 2007-2012 Sergio Pistone\n&copy; 2013-2016 Mladen Milinković",
+		QStringLiteral("&copy; 2007-2017 Subtitle Composer project"),
 		QString(), // Additional text
-		"https://github.com/maxrd2/subtitlecomposer",
-		"maxrd2@smoothware.net");
+		QStringLiteral("https://github.com/maxrd2/subtitlecomposer"),
+		QStringLiteral("maxrd2@smoothware.net"));
 
-	aboutData.addAuthor(i18n("Mladen Milinković"), i18n("Author & Maintainer"), "Mladen Milinkovic <maxrd2@smoothware.net>");
-	aboutData.addAuthor(i18n("Sergio Pistone"), i18n("Original Author"), "Sergio Pistone <sergio_pistone@yahoo.com.ar>");
+	aboutData.addAuthor(i18n("Mladen Milinković"), i18n("Current Author & Maintainer"), "maxrd2@smoothware.net");
+	aboutData.addAuthor(i18n("Sergio Pistone"), i18n("Original Author"), "sergio_pistone@yahoo.com.ar");
 
 	aboutData.setTranslator(i18nc("NAME OF TRANSLATORS", "Your names"), i18nc("EMAIL OF TRANSLATORS", "Your emails"));
 
@@ -78,7 +80,7 @@ main(int argc, char **argv)
 	aboutData.addCredit("Yuri Chornoivan", i18n("Translator"));
 	aboutData.addCredit("Alexandros Perdikomatis", i18n("Translator"));
 	aboutData.addCredit("Barcza Károly", i18n("Translator"));
-	aboutData.addCredit("Martin Steghöfer");
+	aboutData.addCredit("Martin Steghöfer", i18n("code contributions"));
 	aboutData.addCredit("Marius Kittler", i18n("code contributions, Arch Linux packaging, German translation"));
 
 	aboutData.addCredit(i18n("All people who have contributed and I have forgotten to mention"));
