@@ -1,6 +1,6 @@
-/**
+/*
  * Copyright (C) 2007-2009 Sergio Pistone <sergio_pistone@yahoo.com.ar>
- * Copyright (C) 2010-2015 Mladen Milinkovic <max@smoothware.net>
+ * Copyright (C) 2010-2017 Mladen Milinkovic <max@smoothware.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,7 +61,8 @@ PlayerWidget::PlayerWidget(QWidget *parent) :
 	m_translationMode(false),
 	m_showTranslation(false),
 	m_overlayLine(0),
-	m_playingLine(0),
+	m_playingLine(nullptr),
+	m_pauseAfterPlayingLine(nullptr),
 	m_fullScreenTID(0),
 	m_fullScreenMode(false),
 	m_player(VideoPlayer::instance()),
@@ -112,9 +113,13 @@ PlayerWidget::PlayerWidget(QWidget *parent) :
 	fpsTagLabel->setText(i18n("<b>FPS</b>"));
 	fpsTagLabel->installEventFilter(this);
 	m_fpsLabel = new QLabel(m_infoControlsGroupBox);
-
 	m_fpsLabel->setMinimumWidth(m_positionEdit->sizeHint().width());        // sets the minimum width for the whole group
 
+	QLabel *rateTagLabel = new QLabel(m_infoControlsGroupBox);
+	rateTagLabel->setText(i18n("<b>Playback Rate</b>"));
+	rateTagLabel->installEventFilter(this);
+	m_rateLabel = new QLabel(m_infoControlsGroupBox);
+
 	m_volumeSlider = new PointingSlider(Qt::Vertical, this);
 	m_volumeSlider->setFocusPolicy(Qt::NoFocus);
 	m_volumeSlider->setTickPosition(QSlider::NoTicks);
@@ -140,7 +145,10 @@ PlayerWidget::PlayerWidget(QWidget *parent) :
 	videoControlsLayout->addWidget(createToolButton(this, ACT_SET_CURRENT_LINE_HIDE_TIME, 16), 0, 9);
 	videoControlsLayout->addItem(new QSpacerItem(2, 2), 0, 10);
 	videoControlsLayout->addWidget(createToolButton(this, ACT_CURRENT_LINE_FOLLOWS_VIDEO, 16), 0, 11);
-	videoControlsLayout->addWidget(m_seekSlider, 0, 12);
+	videoControlsLayout->addItem(new QSpacerItem(2, 2), 0, 12);
+	videoControlsLayout->addWidget(createToolButton(this, ACT_PLAY_RATE_DECREASE, 16), 0, 13);
+	videoControlsLayout->addWidget(createToolButton(this, ACT_PLAY_RATE_INCREASE, 16), 0, 14);
+	videoControlsLayout->addWidget(m_seekSlider, 0, 15);
 
 	QGridLayout *audioControlsLayout = new QGridLayout();
 	audioControlsLayout->setMargin(0);
@@ -152,12 +160,15 @@ PlayerWidget::PlayerWidget(QWidget *parent) :
 	infoControlsLayout->addWidget(fpsTagLabel, 0, 0);
 	infoControlsLayout->addWidget(m_fpsLabel, 1, 0);
 	infoControlsLayout->addItem(new QSpacerItem(1, 1, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding), 2, 0);
-	infoControlsLayout->addWidget(lengthTagLabel, 3, 0);
-	infoControlsLayout->addWidget(m_lengthLabel, 4, 0);
+	infoControlsLayout->addWidget(rateTagLabel, 3, 0);
+	infoControlsLayout->addWidget(m_rateLabel, 4, 0);
 	infoControlsLayout->addItem(new QSpacerItem(1, 1, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding), 5, 0);
-	infoControlsLayout->addWidget(positionTagLabel, 6, 0);
-	infoControlsLayout->addWidget(m_positionLabel, 7, 0);
-	infoControlsLayout->addWidget(m_positionEdit, 8, 0);
+	infoControlsLayout->addWidget(lengthTagLabel, 6, 0);
+	infoControlsLayout->addWidget(m_lengthLabel, 7, 0);
+	infoControlsLayout->addItem(new QSpacerItem(1, 1, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding), 8, 0);
+	infoControlsLayout->addWidget(positionTagLabel, 9, 0);
+	infoControlsLayout->addWidget(m_positionLabel, 10, 0);
+	infoControlsLayout->addWidget(m_positionEdit, 11, 0);
 
 	m_mainLayout = new QGridLayout(this);
 	m_mainLayout->setMargin(0);
@@ -212,6 +223,9 @@ PlayerWidget::PlayerWidget(QWidget *parent) :
 	fullScreenControlsLayout->addWidget(createToolButton(m_fullScreenControls, ACT_SEEK_TO_PREVIOUS_LINE, FS_BUTTON_SIZE));
 	fullScreenControlsLayout->addWidget(createToolButton(m_fullScreenControls, ACT_SEEK_TO_NEXT_LINE, FS_BUTTON_SIZE));
 	fullScreenControlsLayout->addSpacing(3);
+	fullScreenControlsLayout->addWidget(createToolButton(m_fullScreenControls, ACT_PLAY_RATE_DECREASE, FS_BUTTON_SIZE));
+	fullScreenControlsLayout->addWidget(createToolButton(m_fullScreenControls, ACT_PLAY_RATE_INCREASE, FS_BUTTON_SIZE));
+	fullScreenControlsLayout->addSpacing(3);
 	fullScreenControlsLayout->addWidget(m_fsSeekSlider, 9);
 //  fullScreenControlsLayout->addSpacing( 1 );
 	fullScreenControlsLayout->addWidget(m_fsPositionLabel);
@@ -246,6 +260,7 @@ PlayerWidget::PlayerWidget(QWidget *parent) :
 	connect(m_player, SIGNAL(positionChanged(double)), this, SLOT(onPlayerPositionChanged(double)));
 	connect(m_player, SIGNAL(lengthChanged(double)), this, SLOT(onPlayerLengthChanged(double)));
 	connect(m_player, SIGNAL(framesPerSecondChanged(double)), this, SLOT(onPlayerFramesPerSecondChanged(double)));
+	connect(m_player, SIGNAL(playbackRateChanged(double)), this, SLOT(onPlayerPlaybackRateChanged(double)));
 	connect(m_player, SIGNAL(volumeChanged(double)), this, SLOT(onPlayerVolumeChanged(double)));
 
 	connect(m_player, SIGNAL(leftClicked(const QPoint &)), this, SLOT(onPlayerLeftClicked(const QPoint &)));
@@ -384,6 +399,8 @@ PlayerWidget::plugActions()
 	toolButton(this, ACT_CURRENT_LINE_FOLLOWS_VIDEO)->setDefaultAction(app()->action(ACT_CURRENT_LINE_FOLLOWS_VIDEO));
 	toolButton(this, ACT_TOGGLE_MUTED)->setDefaultAction(app()->action(ACT_TOGGLE_MUTED));
 	toolButton(this, ACT_TOGGLE_FULL_SCREEN)->setDefaultAction(app()->action(ACT_TOGGLE_FULL_SCREEN));
+	toolButton(this, ACT_PLAY_RATE_DECREASE)->setDefaultAction(app()->action(ACT_PLAY_RATE_DECREASE));
+	toolButton(this, ACT_PLAY_RATE_INCREASE)->setDefaultAction(app()->action(ACT_PLAY_RATE_INCREASE));
 
 	toolButton(m_fullScreenControls, ACT_STOP)->setDefaultAction(app()->action(ACT_STOP));
 	toolButton(m_fullScreenControls, ACT_PLAY_PAUSE)->setDefaultAction(app()->action(ACT_PLAY_PAUSE));
@@ -393,6 +410,8 @@ PlayerWidget::plugActions()
 	toolButton(m_fullScreenControls, ACT_SEEK_TO_NEXT_LINE)->setDefaultAction(app()->action(ACT_SEEK_TO_NEXT_LINE));
 	toolButton(m_fullScreenControls, ACT_TOGGLE_MUTED)->setDefaultAction(app()->action(ACT_TOGGLE_MUTED));
 	toolButton(m_fullScreenControls, ACT_TOGGLE_FULL_SCREEN)->setDefaultAction(app()->action(ACT_TOGGLE_FULL_SCREEN));
+	toolButton(m_fullScreenControls, ACT_PLAY_RATE_DECREASE)->setDefaultAction(app()->action(ACT_PLAY_RATE_DECREASE));
+	toolButton(m_fullScreenControls, ACT_PLAY_RATE_INCREASE)->setDefaultAction(app()->action(ACT_PLAY_RATE_INCREASE));
 }
 
 void
@@ -487,7 +506,7 @@ PlayerWidget::setSubtitle(Subtitle *subtitle)
 		m_subtitle = 0;                 // has to be set to 0 for invalidateOverlayLine
 
 		invalidateOverlayLine();
-		setPlayingLine(0);
+		setPlayingLine(nullptr);
 	}
 
 	m_subtitle = subtitle;
@@ -514,7 +533,7 @@ PlayerWidget::setShowTranslation(bool showTranslation)
 		m_showTranslation = showTranslation;
 
 		invalidateOverlayLine();
-		setPlayingLine(0);
+		setPlayingLine(nullptr);
 	}
 }
 
@@ -545,7 +564,6 @@ PlayerWidget::updateOverlayLine(const Time &videoPosition)
 			if(videoPosition >= m_overlayLine->showTime()) { // m_overlayLine is the line to show
 				const SString &text = m_showTranslation ? m_overlayLine->secondaryText() : m_overlayLine->primaryText();
 				m_textOverlay->setText(text.richString(SString::Verbose));
-				setPlayingLine(m_overlayLine);
 			}
 			return;
 		} else {
@@ -567,15 +585,56 @@ PlayerWidget::updateOverlayLine(const Time &videoPosition)
 				if(m_overlayLine->showTime() <= videoPosition && videoPosition <= m_overlayLine->hideTime()) {
 					const SString &text = m_showTranslation ? m_overlayLine->secondaryText() : m_overlayLine->primaryText();
 					m_textOverlay->setText(text.richString(SString::Verbose));
-					setPlayingLine(m_overlayLine);
 				}
 				return;
 			}
 		}
-		setPlayingLine(0);
 	}
 }
 
+void
+PlayerWidget::updatePlayingLine(const Time &videoPosition)
+{
+	if(!m_subtitle)
+		return;
+
+	// playing line is already correct
+	if(m_playingLine && videoPosition <= m_playingLine->hideTime() && videoPosition >= m_playingLine->showTime())
+		return;
+
+	// pause if requested
+	if(m_pauseAfterPlayingLine && videoPosition >= m_pauseAfterPlayingLine->hideTime()) {
+		m_player->pause();
+		setPlayingLine(const_cast<SubtitleLine *>(m_pauseAfterPlayingLine));
+		m_pauseAfterPlayingLine = nullptr;
+		return;
+	}
+
+	// overlay line is playing line
+	if(m_overlayLine && videoPosition <= m_overlayLine->hideTime() && videoPosition >= m_overlayLine->showTime()) {
+		setPlayingLine(m_overlayLine);
+		return;
+	}
+
+	// iterate through all lines and find the playing line
+	SubtitleIterator it(*m_subtitle);
+	while(it.current()) {
+		if(videoPosition <= it.current()->hideTime() && videoPosition >= it.current()->showTime()) {
+			setPlayingLine(it.current());
+			return;
+		}
+		++it;
+	}
+
+	setPlayingLine(nullptr);
+}
+
+void
+PlayerWidget::pauseAfterPlayingLine(const SubtitleLine *line)
+{
+	m_pauseAfterPlayingLine = line;
+}
+
 void
 PlayerWidget::invalidateOverlayLine()
 {
@@ -658,6 +717,7 @@ PlayerWidget::onSeekSliderValueChanged(int value)
 {
 	if(m_updateVideoPosition) {
 		m_updatePositionControls = MAGIC_NUMBER;
+		pauseAfterPlayingLine(nullptr);
 		m_player->seek(m_player->length() * value / 1000.0, true);
 	}
 }
@@ -665,6 +725,7 @@ PlayerWidget::onSeekSliderValueChanged(int value)
 void
 PlayerWidget::onSeekSliderMoved(int value)
 {
+	pauseAfterPlayingLine(nullptr);
 	m_player->seek(m_player->length() * value / 1000.0, false);
 
 	Time time((long)(m_player->length() * value));
@@ -681,6 +742,7 @@ PlayerWidget::onPositionEditValueChanged(int position)
 {
 	if(m_positionEdit->hasFocus()) {
 		m_updatePositionControls = MAGIC_NUMBER;
+		pauseAfterPlayingLine(nullptr);
 		m_player->seek(position / 1000.0, true);
 	}
 }
@@ -716,6 +778,7 @@ PlayerWidget::onPlayerFileOpened(const QString & /*filePath */)
 	m_positionLabel->setText(i18n("<i>Unknown</i>"));
 	m_lengthLabel->setText(i18n("<i>Unknown</i>"));
 	m_fpsLabel->setText(i18n("<i>Unknown</i>"));
+	m_rateLabel->setText(i18n("<i>Unknown</i>"));
 
 	m_lengthString = UNKNOWN_LENGTH_STRING;
 
@@ -777,7 +840,7 @@ PlayerWidget::onPlayerPositionChanged(double seconds)
 {
 	if(m_updatePositionControls > 0) {
 		if(seconds >= 0) {
-			Time videoPosition((long)(seconds * 1000));
+			Time videoPosition(seconds * 1000.);
 
 			m_positionLabel->setText(videoPosition.toString());
 			m_fsPositionLabel->setText(videoPosition.toString(false) + m_lengthString);
@@ -786,6 +849,7 @@ PlayerWidget::onPlayerPositionChanged(double seconds)
 				m_positionEdit->setValue(videoPosition.toMillis());
 
 			updateOverlayLine(videoPosition);
+			updatePlayingLine(videoPosition);
 
 			int sliderValue = (int)((seconds / m_player->length()) * 1000);
 
@@ -797,8 +861,9 @@ PlayerWidget::onPlayerPositionChanged(double seconds)
 			m_positionLabel->setText(i18n("<i>Unknown</i>"));
 			m_fsPositionLabel->setText(Time().toString(false) + m_lengthString);
 		}
-	} else if(m_updatePositionControls < 0)
+	} else if(m_updatePositionControls < 0) {
 		m_updatePositionControls += 2;
+	}
 }
 
 void
@@ -819,6 +884,12 @@ PlayerWidget::onPlayerFramesPerSecondChanged(double fps)
 	m_fpsLabel->setText(fps > 0 ? QString::number(fps, 'f', 3) : i18n("<i>Unknown</i>"));
 }
 
+void
+PlayerWidget::onPlayerPlaybackRateChanged(double rate)
+{
+	m_rateLabel->setText(rate > .0 ? QStringLiteral("%1x").arg(rate, 0, 'g', 3) : i18n("<i>Unknown</i>"));
+}
+
 void
 PlayerWidget::onPlayerStopped()
 {
@@ -827,7 +898,7 @@ PlayerWidget::onPlayerStopped()
 	m_seekSlider->setEnabled(false);
 	m_fsSeekSlider->setEnabled(false);
 
-	setPlayingLine(0);
+	setPlayingLine(nullptr);
 
 	updatePositionEditVisibility();
 }
@@ -877,6 +948,11 @@ PlayerWidget::onPlayerRightClicked(const QPoint &point)
 
 	menu->addSeparator();
 
+	menu->addAction(app()->action(ACT_PLAY_RATE_DECREASE));
+	menu->addAction(app()->action(ACT_PLAY_RATE_INCREASE));
+
+	menu->addSeparator();
+
 	menu->addAction(app()->action(ACT_SET_ACTIVE_AUDIO_STREAM));
 	menu->addAction(app()->action(ACT_INCREASE_VOLUME));
 	menu->addAction(app()->action(ACT_DECREASE_VOLUME));
