@@ -1,5 +1,5 @@
-/**
- * Copyright (C) 2010-2016 Mladen Milinkovic <max@smoothware.net>
+/*
+ * Copyright (C) 2010-2017 Mladen Milinkovic <max@smoothware.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,10 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "speechprocessor.h"
 #include "speechplugin.h"
 #include "application.h"
