@@ -1,5 +1,5 @@
-/**
- * Copyright (C) 2010-2016 Mladen Milinkovic <max@smoothware.net>
+/*
+ * Copyright (C) 2010-2017 Mladen Milinkovic <max@smoothware.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -267,10 +267,11 @@ StreamProcessor::onPadAdded(GstElement */*decodebin*/, GstPad *pad, gpointer use
 			// link decodebin to textsink
 			const gchar *padName = gst_pad_get_name(pad);
 			GstCaps *outputFilter = GStreamer::textCapsFromEncoding("utf8");
-			if(GST_PAD_LINK_FAILED(GStreamer::link(GST_BIN(me->m_decodingPipeline), "decodebin", padName, "textsink", "sink", outputFilter)))
+			if(GST_PAD_LINK_FAILED(GStreamer::link(GST_BIN(me->m_decodingPipeline), "decodebin", padName, "textsink", "sink", outputFilter))) {
 				outputFilter = GStreamer::textCapsFromEncoding(NULL);
-			if(GST_PAD_LINK_FAILED(GStreamer::link(GST_BIN(me->m_decodingPipeline), "decodebin", padName, "textsink", "sink", outputFilter)))
-				qCritical() << "Failed to connect decodebin pad" << padName;
+				if(GST_PAD_LINK_FAILED(GStreamer::link(GST_BIN(me->m_decodingPipeline), "decodebin", padName, "textsink", "sink", outputFilter)))
+					qCritical() << "Failed to connect decodebin pad" << padName;
+			}
 			qDebug() << "Selected text stream #" << me->m_audioStreamCurrent << " [" << padName << "] " << gst_caps_to_string(caps);
 		}
 	}
@@ -326,7 +327,7 @@ StreamProcessor::decoderMessageProc()
 
 	gint64 time;
 	if(gst_element_query_position(GST_ELEMENT(m_decodingPipeline), GST_FORMAT_TIME, &time)) {
-		if(m_streamPos != (quint64)time / GST_MSECOND) {
+		if(m_streamPos != static_cast<quint64>(time) / GST_MSECOND) {
 			m_streamPos = time / GST_MSECOND;
 			if(m_streamLen)
 				emit streamProgress(m_streamPos, m_streamLen);
