@@ -1,6 +1,6 @@
-/**
+/*
  * Copyright (C) 2007-2009 Sergio Pistone <sergio_pistone@yahoo.com.ar>
- * Copyright (C) 2010-2016 Mladen Milinkovic <max@smoothware.net>
+ * Copyright (C) 2010-2017 Mladen Milinkovic <max@smoothware.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,10 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "videoplayer.h"
 #include "playerbackend.h"
 
@@ -67,6 +70,8 @@ protected:
 
 	virtual bool stop() { return false; }
 
+	virtual void playbackRate(double /*newRate*/) { }
+
 	virtual bool setActiveAudioStream(int /*audioStream*/) { return false; }
 
 	virtual bool setVolume(double /*volume*/) { return false; }
@@ -91,6 +96,7 @@ VideoPlayer::VideoPlayer() :
 	m_savedPosition(-1.0),
 	m_length(-1.0),
 	m_framesPerSecond(-1.0),
+	m_playbackRate(.0),
 	m_minPositionDelta(DEFAULT_MIN_POSITION_DELTA),
 	m_textStreams(),
 	m_activeAudioStream(-1),
@@ -407,13 +413,36 @@ VideoPlayer::resetState()
 }
 
 void
-VideoPlayer::setPosition(double position)
+VideoPlayer::notifyVolume(double volume)
+{
+	if(volume < 0.0)
+		volume = 0.0;
+	else if(volume > 100.0)
+		volume = 100.0;
+
+	if(m_volume != volume) {
+		m_volume = volume;
+		emit volumeChanged(m_volume);
+	}
+}
+
+void
+VideoPlayer::notifyMute(bool muted)
+{
+	if(m_muted != muted) {
+		m_muted = muted;
+		emit muteChanged(m_muted);
+	}
+}
+
+void
+VideoPlayer::notifyPosition(double position)
 {
 	if(m_state <= VideoPlayer::Closed)
 		return;
 
 	if(position > m_length && m_length > 0)
-		setLength(position);
+		notifyLength(position);
 
 	if(m_position != position) {
 		if(m_position <= 0 || m_minPositionDelta <= 0.0) {
@@ -430,7 +459,7 @@ VideoPlayer::setPosition(double position)
 }
 
 void
-VideoPlayer::setLength(double length)
+VideoPlayer::notifyLength(double length)
 {
 	if(m_state <= VideoPlayer::Closed)
 		return;
@@ -442,7 +471,7 @@ VideoPlayer::setLength(double length)
 }
 
 void
-VideoPlayer::setFramesPerSecond(double framesPerSecond)
+VideoPlayer::notifyFramesPerSecond(double framesPerSecond)
 {
 	if(m_state <= VideoPlayer::Closed)
 		return;
@@ -455,14 +484,14 @@ VideoPlayer::setFramesPerSecond(double framesPerSecond)
 }
 
 void
-VideoPlayer::setTextStreams(const QStringList &textStreams)
+VideoPlayer::notifyTextStreams(const QStringList &textStreams)
 {
 	m_textStreams = textStreams;
 	emit textStreamsChanged(m_textStreams);
 }
 
 void
-VideoPlayer::setAudioStreams(const QStringList &audioStreams, int activeAudioStream)
+VideoPlayer::notifyAudioStreams(const QStringList &audioStreams, int activeAudioStream)
 {
 	if(m_state <= VideoPlayer::Closed)
 		return;
@@ -480,7 +509,7 @@ VideoPlayer::setAudioStreams(const QStringList &audioStreams, int activeAudioStr
 }
 
 void
-VideoPlayer::setState(VideoPlayer::State newState)
+VideoPlayer::notifyState(VideoPlayer::State newState)
 {
 	if(m_state == VideoPlayer::Opening) {
 		if(newState == VideoPlayer::Playing) {
@@ -495,6 +524,7 @@ VideoPlayer::setState(VideoPlayer::State newState)
 			// we emit this signals in case their values were already set
 			emit lengthChanged(m_length);
 			emit framesPerSecondChanged(m_framesPerSecond);
+			emit playbackRateChanged(m_playbackRate);
 			emit textStreamsChanged(m_textStreams);
 			emit audioStreamsChanged(m_audioStreams);
 			emit activeAudioStreamChanged(m_activeAudioStream);
@@ -525,7 +555,7 @@ VideoPlayer::setState(VideoPlayer::State newState)
 }
 
 void
-VideoPlayer::setErrorState(const QString &errorMessage)
+VideoPlayer::notifyErrorState(const QString &errorMessage)
 {
 	if(m_state < VideoPlayer::Opening)
 		return;
@@ -742,6 +772,15 @@ VideoPlayer::setActiveAudioStream(int audioStreamIndex)
 	return true;
 }
 
+void
+VideoPlayer::playbackRate(double newRate)
+{
+	if(m_state != VideoPlayer::Playing || newRate < .125 || newRate > 128)
+		return;
+
+	activeBackend()->playbackRate(newRate);
+}
+
 void
 VideoPlayer::increaseVolume(double amount)
 {
