@@ -1,6 +1,6 @@
-/**
+/*
  * Copyright (C) 2007-2009 Sergio Pistone <sergio_pistone@yahoo.com.ar>
- * Copyright (C) 2010-2015 Mladen Milinkovic <max@smoothware.net>
+ * Copyright (C) 2010-2017 Mladen Milinkovic <max@smoothware.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -117,6 +117,7 @@ MPVBackend::mpvInit()
 
 	// Receive property change events with MPV_EVENT_PROPERTY_CHANGE
 	mpv_observe_property(m_mpv, 0, "time-pos", MPV_FORMAT_DOUBLE);
+	mpv_observe_property(m_mpv, 0, "speed", MPV_FORMAT_DOUBLE);
 	mpv_observe_property(m_mpv, 0, "pause", MPV_FORMAT_FLAG);
 	mpv_observe_property(m_mpv, 0, "length", MPV_FORMAT_DOUBLE);
 	mpv_observe_property(m_mpv, 0, "track-list", MPV_FORMAT_NODE);
@@ -171,14 +172,14 @@ MPVBackend::mpvEventHandle(mpv_event *event)
 					setPlayerState(VideoPlayer::Playing);
 				}
 			}
-		} else if(strcmp(prop->name, "length") == 0) {
-			if(prop->format == MPV_FORMAT_DOUBLE) {
-				double duration = *(double *)prop->data;
-				setPlayerLength(duration);
-			}
 		} else if(strcmp(prop->name, "track-list") == 0) {
 			updateAudioData(prop);
 			updateTextData(prop);
+		} else if(strcmp(prop->name, "speed") == 0) {
+			if(prop->format == MPV_FORMAT_DOUBLE) {
+				double rate = *(double *)prop->data;
+				playbackRateNotify(rate);
+			}
 		}
 		break;
 	}
@@ -283,16 +284,19 @@ MPVBackend::updateVideoData()
 {
 	// Retrieve the new video size.
 	int64_t w, h;
-	double dar, fps;
+	double dar, fps, length;
 	if(mpv_get_property(m_mpv, "dwidth", MPV_FORMAT_INT64, &w) >= 0
 			&& mpv_get_property(m_mpv, "dheight", MPV_FORMAT_INT64, &h) >= 0
 			&& mpv_get_property(m_mpv, "video-aspect", MPV_FORMAT_DOUBLE, &dar)
 			&& w > 0 && h > 0) {
 		player()->videoWidget()->setVideoResolution(w, h, dar);
 	}
-	if(mpv_get_property(m_mpv, "fps", MPV_FORMAT_DOUBLE, &fps) >= 0 && fps > 0) {
+	if(mpv_get_property(m_mpv, "estimated-vf-fps", MPV_FORMAT_DOUBLE, &fps) >= 0 && fps > 0)
 		setPlayerFramesPerSecond(fps);
-	}
+	else if(mpv_get_property(m_mpv, "container-fps", MPV_FORMAT_DOUBLE, &fps) >= 0 && fps > 0)
+		setPlayerFramesPerSecond(fps);
+	if(mpv_get_property(m_mpv, "duration", MPV_FORMAT_DOUBLE, &length) >= 0 && length > 0)
+		setPlayerLength(length);
 }
 
 
@@ -390,6 +394,16 @@ MPVBackend::seek(double seconds, bool accurate)
 	return true;
 }
 
+/*virtual*/ void
+MPVBackend::playbackRate(double newRate)
+{
+	if(newRate > 1.) // without frame dropping we might go out of sync
+		mpv_set_option_string(m_mpv, "framedrop", "vo");
+	else
+		mpv_set_option_string(m_mpv, "framedrop", SCConfig::mpvFrameDropping() ? "vo" : "no");
+	mpv_set_option(m_mpv, "speed", MPV_FORMAT_DOUBLE, &newRate);
+}
+
 bool
 MPVBackend::setActiveAudioStream(int audioStream)
 {
@@ -423,8 +437,18 @@ MPVBackend::reconfigure()
 	if(!m_mpv)
 		return false;
 
-	if(SCConfig::mpvVideoOutputEnabled())
+	if(SCConfig::mpvVideoOutputEnabled()) {
+#if MPV_CLIENT_API_VERSION >= MPV_MAKE_VERSION(1, 21)
+		if(SCConfig::mpvVideoOutput() == QStringLiteral("opengl-hq")) {
+			mpv_set_option_string(m_mpv, "vo", "opengl");
+			mpv_set_option_string(m_mpv, "profile", "opengl-hq");
+		} else {
+			mpv_set_option_string(m_mpv, "vo", SCConfig::mpvVideoOutput().toUtf8().constData());
+		}
+#else
 		mpv_set_option_string(m_mpv, "vo", SCConfig::mpvVideoOutput().toUtf8().constData());
+#endif
+	}
 
 	mpv_set_option_string(m_mpv, "hwdec", SCConfig::mpvHwDecodeEnabled() ? SCConfig::mpvHwDecode().toUtf8().constData() : "no");
 
@@ -450,10 +474,14 @@ MPVBackend::reconfigure()
 		mpv_set_option_string(m_mpv, "drc", "1:0.25");
 
 	if(SCConfig::mpvVolumeAmplificationEnabled()) {
+#if MPV_CLIENT_API_VERSION >= MPV_MAKE_VERSION(1, 22)
+		mpv_set_option_string(m_mpv, "volume-max", QString::number(SCConfig::mpvVolumeAmplification()).toUtf8().constData());
+#else
 		mpv_set_option_string(m_mpv, "softvol", "yes");
 		mpv_set_option_string(m_mpv, "softvol-max", QString::number(SCConfig::mpvVolumeAmplification()).toUtf8().constData());
 	} else {
 		mpv_set_option_string(m_mpv, "softvol", "no");
+#endif
 	}
 
 	// restart playing
